#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import datetime
import mozinfo
import sys
from optparse import OptionParser
from inboundfinder import getInboundRevisions
from runnightly import NightlyRunner, parseBits
from runinbound import InboundRunner
from utils import strsplit, get_date

class Bisector(object):

    currDate = ''
    foundRepo = None

    def __init__(self, nightlyRunner, inboundRunner, appname="firefox",
                 lastGoodRevision=None, firstBadRevision=None):
        self.nightlyRunner = nightlyRunner
        self.inboundRunner = inboundRunner
        self.appname = appname
        self.lastGoodRevision = lastGoodRevision
        self.firstBadRevision = firstBadRevision

    def findRegressionChset(self, lastGoodRevision, firstBadRevision):
        #Uses mozcommitbuilder to bisect on changesets
        #Only needed if they want to bisect, so we'll put the dependency here.
        from mozcommitbuilder import builder
        commitBuilder = builder.Builder()

        print "\n Narrowed changeset range from " + lastGoodRevision + " to " + firstBadRevision +"\n"

        print "Time to do some bisecting and building!"
        commitBuilder.bisect(lastGoodRevision, firstBadRevision)
        quit()

    def offer_build(self, lastGoodRevision, firstBadRevision):
        verdict = raw_input("do you want to bisect further by fetching the repository and building? (y or n) ")
        if verdict != "y":
            sys.exit()

        if self.appname == "firefox":
            self.findRegressionChset(lastGoodRevision, firstBadRevision)
        else:
            print "Bisection on anything other than firefox is not currently supported."
            sys.exit()

    def printRange(self, goodDate=None, badDate=None):
        def _printDateRevision(name, revision, date):
            if date and revision:
                print "%s revision: %s (%s)" % (name, revision, date)
            elif revision:
                print "%s revision: %s" % (name, revision)
            elif date:
                print "%s build: %s" % (name, date)
        _printDateRevision("Last good", self.lastGoodRevision, goodDate)
        _printDateRevision("First bad", self.firstBadRevision, badDate)

        print "Pushlog:\n" + self.getPushlogUrl(goodDate, badDate) + "\n"

    def _ensureMetadata(self, goodDate, badDate):
        print "Ensuring we have enough metadata to get a pushlog..."
        if not self.lastGoodRevision:
            self.nightlyRunner.install(goodDate)
            (self.foundRepo, self.lastGoodRevision) = \
                self.nightlyRunner.getAppInfo()
        if not self.firstBadRevision:
            self.nightlyRunner.install(badDate)
            (self.foundRepo, self.firstBadRevision) = \
                self.nightlyRunner.getAppInfo()

    def _get_verdict(self, buildType, offerSkip=True):
        verdict = ""
        options = ['good','g','bad','b','retry','r']
        if offerSkip:
            options += ['skip','s']
        options += ['exit']
        while verdict not in options:
            verdict = raw_input("Was this %s build good, bad, or broken? (type 'good', 'bad', 'skip', 'retry', or 'exit' and press Enter): " % buildType)

        # shorten verdict to one character for processing...
        if len(verdict) > 1:
            return verdict[0]

        return verdict

    def bisect_inbound(self, inboundRevisions=None):
        if not inboundRevisions:
            print "Getting inbound builds between %s and %s" % (
                self.lastGoodRevision, self.firstBadRevision)
            inboundRevisions = getInboundRevisions(
                self.lastGoodRevision, self.firstBadRevision,
                appName=self.inboundRunner.appName,
                bits=self.inboundRunner.bits)

            if not inboundRevisions:
                print "Oh noes, no (more) inbound revisions :("
                self.offer_build(self.lastGoodRevision,
                                 self.firstBadRevision)
                return
        # hardcode repo to mozilla-central (if we use inbound, we may be
        # missing some revisions that went into the nightlies which we may
        # also be comparing against...)

        mid = len(inboundRevisions) / 2
        print "Testing inbound build with timestamp %s, revision %s" % (inboundRevisions[mid][1],
                                                                        inboundRevisions[mid][0])
        self.inboundRunner.start(inboundRevisions[mid][1])

        verdict = self._get_verdict('inbound', offerSkip=False)
        self.inboundRunner.stop()
        self.foundRepo = self.inboundRunner.getAppInfo()[0]
        if verdict == 'g':
            self.lastGoodRevision = self.inboundRunner.getAppInfo()[1]
        elif verdict == 'b':
            self.firstBadRevision = self.inboundRunner.getAppInfo()[1]
        elif verdict == 'r':
            # do the same thing over again
            self.bisect_inbound(inboundRevisions=inboundRevisions)
            return
        elif verdict == 'e':
            print 'Newest known good inbound revision: %s' % self.lastGoodRevision
            print 'Oldest known bad inbound revision: %s' % self.firstBadRevision
            print 'To resume, run:'
            print 'mozregression --inbound --good-rev=%s --bad-rev=%s' % (
                self.lastGoodRevision, self.firstBadRevision)
            return

        if len(inboundRevisions) > 1 and verdict == 'g':
            self.bisect_inbound(inboundRevisions[(mid+1):])
        elif len(inboundRevisions) > 1 and verdict == 'b':
            self.bisect_inbound(inboundRevisions[:mid])
        else:
            # no more inbounds to be bisect, we must build
            print "No more inbounds to bisect"
            self.printRange()
            self.offer_build(self.lastGoodRevision, self.firstBadRevision)

    def bisect_nightlies(self, goodDate, badDate, skips=0):
        midDate = goodDate + (badDate - goodDate) / 2

        midDate += datetime.timedelta(days=skips)

        if midDate == badDate or midDate == goodDate:
            print "Got as far as we can go bisecting nightlies..."
            self._ensureMetadata(goodDate, badDate)
            if self.appname == 'firefox' or self.appname == 'fennec':
                self.printRange(goodDate, badDate)
                print "... attempting to bisect inbound builds"
                self.bisect_inbound()
                return
            else:
                print "(no more options with %s)" % self.appname
                sys.exit()

        # run the nightly from that date
        print midDate
        dest = self.nightlyRunner.start(midDate)
        while not dest:
            midDate += datetime.timedelta(days=1)
            if midDate == badDate:
                self.printRange(goodDate, badDate)
            dest = self.nightlyRunner.start(midDate)

        self.prevDate = self.currDate
        self.currDate = midDate

        verdict = self._get_verdict('nightly')
        self.nightlyRunner.stop()
        self.foundRepo = self.nightlyRunner.getAppInfo()[0]
        if verdict == 'g':
            self.lastGoodRevision = self.nightlyRunner.getAppInfo()[1]
            self.bisect_nightlies(midDate, badDate)
        elif verdict == 'b':
            self.firstBadRevision = self.nightlyRunner.getAppInfo()[1]
            self.bisect_nightlies(goodDate, midDate)
        elif verdict == 's':
            #skip -- go 1 day further down
            self.bisect_nightlies(goodDate, badDate, skips=skips+1)
        elif verdict == 'e':
            self.nightlyRunner.stop()
            goodDateString = '%04d-%02d-%02d' % (goodDate.year, goodDate.month, goodDate.day)
            badDateString = '%04d-%02d-%02d' % (badDate.year, badDate.month, badDate.day)
            print 'Newest known good nightly: %s' % goodDateString
            print 'Oldest known bad nightly: %s' % badDateString
            print 'To resume, run:'
            print 'mozregression --good=%s --bad=%s' % (goodDateString, badDateString)
            return
        else:
            #retry -- since we're just calling ourselves with the same parameters, it does the same thing again
            self.bisect_nightlies(goodDate, badDate)

    def getPushlogUrl(self, goodDate, badDate):
        # if we don't have precise revisions, we need to resort to just
        # using handwavey dates
        if not self.lastGoodRevision or not self.firstBadRevision:
            # pushlogs are typically done with the oldest date first
            if goodDate < badDate:
                start = goodDate
                end = badDate
            else:
                start = badDate
                end = goodDate
            return "%s/pushloghtml?startdate=%s&enddate=%s" % (self.foundRepo,
                                                               start, end)

        return "%s/pushloghtml?fromchange=%s&tochange=%s" % (
            self.foundRepo, self.lastGoodRevision, self.firstBadRevision)

def cli():
    parser = OptionParser()
    parser.add_option("-b", "--bad", dest="bad_date",help="first known bad nightly build, default is today",
                      metavar="YYYY-MM-DD", default=str(datetime.date.today()))
    parser.add_option("-g", "--good", dest="good_date",help="last known good nightly build",
                      metavar="YYYY-MM-DD", default=None)
    parser.add_option("-e", "--addons", dest="addons",help="list of addons to install", metavar="PATH1,PATH2", default="")
    parser.add_option("-p", "--profile", dest="profile", help="profile to use with nightlies", metavar="PATH")
    parser.add_option("-a", "--args", dest="cmdargs", help="command-line arguments to pass to the application",
                      metavar="ARG1,ARG2", default="")
    parser.add_option("-n", "--app", dest="app", help="application name (firefox, fennec or thunderbird)",
                      metavar="[firefox|fennec|thunderbird]", default="firefox")
    parser.add_option("-r", "--repo", dest="repo_name", help="repository name on ftp.mozilla.org",
                      metavar="[tracemonkey|mozilla-1.9.2]", default=None)
    parser.add_option("--bits", dest="bits", help="force 32 or 64 bit version (only applies to x86_64 boxes)",
                      choices=("32","64"), default=mozinfo.bits)
    parser.add_option("--persist", dest="persist", help="the directory in which files are to persist ie. /Users/someuser/Documents")
    parser.add_option("--inbound", action="store_true", dest="inbound", help="use inbound instead of nightlies (use --good-rev and --bad-rev options")
    parser.add_option("--bad-rev", dest="firstBadRevision",help="first known bad revision (use with --inbound)")
    parser.add_option("--good-rev", dest="lastGoodRevision",help="last known good revision (use with --inbound)")

    (options, args) = parser.parse_args()

    options.bits = parseBits(options.bits)

    addons = strsplit(options.addons, ",")
    cmdargs = strsplit(options.cmdargs, ",")

    inboundRunner = None
    if options.app == "firefox" or options.app == "fennec":
        inboundRunner = InboundRunner(appname=options.app,
                                      addons=addons,
                                      repo_name=options.repo_name,
                                      profile=options.profile,
                                      cmdargs=cmdargs, bits=options.bits,
                                      persist=options.persist)

    if options.inbound:
        if not options.lastGoodRevision or not options.firstBadRevision:
            print "If bisecting inbound, both --good-rev and --bad-rev " \
                " must be set"
            sys.exit(1)
        bisector = Bisector(None, inboundRunner, appname=options.app,
                            lastGoodRevision=options.lastGoodRevision,
                            firstBadRevision=options.firstBadRevision)
        bisector.bisect_inbound()
    else:
        if not options.good_date:
            options.good_date = "2009-01-01"
            print "No 'good' date specified, using " + options.good_date
        nightlyRunner = NightlyRunner(appname=options.app, addons=addons,
                                      repo_name=options.repo_name,
                                      profile=options.profile, cmdargs=cmdargs,
                                      bits=options.bits,
                                      persist=options.persist)
        bisector = Bisector(nightlyRunner, inboundRunner, appname=options.app)
        bisector.bisect_nightlies(get_date(options.good_date),
                                  get_date(options.bad_date))


if __name__ == "__main__":
    cli()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import datetime
import os
import re
import sys
import urllib2

from BeautifulSoup import BeautifulSoup

def strsplit(string, sep):
    # XXX https://github.com/mozilla/mozregression/issues/50
    strlist = string.split(sep)
    if strlist == ['']:
      return []
    return strlist

def get_date(dateString):
    p = re.compile('(\d{4})\-(\d{1,2})\-(\d{1,2})')
    m = p.match(dateString)
    if not m:
        print "Incorrect date format"
        return
    return datetime.date(int(m.group(1)), int(m.group(2)), int(m.group(3)))

def update_download_progress(percent):
    sys.stdout.write("===== Downloaded %d%% =====\r"%percent)
    sys.stdout.flush()
    if percent >= 100:
        sys.stdout.write("\n")

def download_url(url, dest=None, message="Downloading build from:"):
    if os.path.exists(dest):
        print "Using local file: %s" % dest
        return

    if message:
        print "%s %s"%(message, url)

    CHUNK = 16 * 1024
    bytes_so_far = 0.0
    tmp_file = dest + ".part"
    r = urllib2.urlopen(url)
    total_size = int(r.info().getheader('Content-length').strip())
    if dest == None:
        dest = os.path.basename(url)
    
    f = open(tmp_file, 'wb')
    # write the file to the tmp_file
    for chunk in iter(lambda: r.read(CHUNK), ''):
        bytes_so_far += CHUNK
        f.write(chunk)
        percent = (bytes_so_far / total_size) * 100
        update_download_progress(percent)
    f.close()
    # move the temp file to the dest
    os.rename(tmp_file, dest)

    return dest

def urlLinks(url):
    r = urllib2.urlopen(url)
    content = r.read()
    if r.getcode() != 200:
        return []

    soup = BeautifulSoup(content)
    # do not return a generator but an array, so we can store it for later use
    return [link for link in soup.findAll('a')]



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import datetime
import os
import mozinstall
import re
import subprocess
import sys
import tempfile
import mozinfo
import zipfile

from mozfile import rmtree
from mozprofile import FirefoxProfile
from mozprofile import ThunderbirdProfile
from mozrunner import Runner
from optparse import OptionParser
from utils import strsplit, get_date, download_url, urlLinks
from ConfigParser import ConfigParser

subprocess._cleanup = lambda : None # mikeal's fix for subprocess threading bug
# XXX please reference this issue with a URL!

class Nightly(object):

    name = None # abstract base class
    _monthlinks = {}
    lastdest = None
    tempdir = None

    @staticmethod
    def _getBuildRegex(bits):
        if mozinfo.os == "win":
            if bits == 64:
                # XXX this should actually throw an error to be consumed by the caller
                print "No builds available for 64 bit Windows (try specifying --bits=32)"
                sys.exit()
            return ".*win32.zip"
        elif mozinfo.os == "linux":
            if bits == 64:
                return ".*linux-x86_64.tar.bz2"
            else:
                return ".*linux-i686.tar.bz2"
        elif mozinfo.os == "mac":
            return ".*mac.*\.dmg"

    def __init__(self, repo_name=None, bits=mozinfo.bits, persist=None):
        self.buildRegex = self._getBuildRegex(bits)
        self.persist = persist
        self.repo_name = repo_name

    ### cleanup functions

    def remove_tempdir(self):
        if self.tempdir:
            rmtree(self.tempdir)
            self.tempdir = None

    def remove_lastdest(self):
        if self.lastdest:
            os.remove(self.lastdest)
            self.lastdest = None

    def cleanup(self):
        self.remove_tempdir()
        if not self.persist:
            self.remove_lastdest()

    __del__ = cleanup

    ### installation functions

    def get_destination(self, url, date):
        repo_name = self.repo_name or self.getRepoName(date)
        dest = os.path.basename(url)
        if self.persist is not None:
            date_str = date.strftime("%Y-%m-%d")
            dest = os.path.join(self.persist, "%s--%s--%s"%(date_str, repo_name, dest))
        return dest

    def download(self, date=datetime.date.today(), dest=None):
        url = self.getBuildUrl(date)
        if url:
            if not dest:
                dest = self.get_destination(url, date)
            if not self.persist:
                self.remove_lastdest()

            self.dest = self.lastdest = dest
            download_url(url, dest)
            return True
        else:
            return False

    def install(self):
        if not self.name:
            raise NotImplementedError("Can't invoke abstract base class")
        self.remove_tempdir()
        self.tempdir = tempfile.mkdtemp()
        self.binary = mozinstall.get_binary(mozinstall.install(src=self.dest, dest=self.tempdir), self.name)
        return True

    def getBuildUrl(self, datestamp):
        if self.appName == 'fennec':
            repo = 'mobile'
        else:
            repo = 'firefox'
        url = "http://ftp.mozilla.org/pub/mozilla.org/" + repo + "/nightly/"
        year = str(datestamp.year)
        month = "%02d" % datestamp.month
        day = "%02d" % datestamp.day
        repo_name = self.repo_name or self.getRepoName(datestamp)
        url += year + "/" + month + "/"

        linkRegex = '^' + year + '-' + month + '-' + day + '-' + '[\d-]+' + repo_name + '/$'
        cachekey = year + '-' + month
        if cachekey in self._monthlinks:
            monthlinks = self._monthlinks[cachekey]
        else:
            monthlinks = urlLinks(url)
            self._monthlinks[cachekey] = monthlinks

        # first parse monthly list to get correct directory
        for dirlink in monthlinks:
            dirhref = dirlink.get("href")
            if re.match(linkRegex, dirhref):
                # now parse the page for the correct build url
                for link in urlLinks(url + dirhref):
                    href = link.get("href")
                    if re.match(self.buildRegex, href):
                        return url + dirhref + href

    ### functions for invoking nightly

    def getAppInfo(self):
        parser = ConfigParser()
        ini_file = os.path.join(os.path.dirname(self.binary), "application.ini")
        parser.read(ini_file)
        try:
            changeset = parser.get('App', 'SourceStamp')
            repo = parser.get('App', 'SourceRepository')
            return (repo, changeset)
        except:
            return None

    def start(self, profile, addons, cmdargs):
        if profile:
            profile = self.profileClass(profile=profile, addons=addons)
        elif len(addons):
            profile = self.profileClass(addons=addons)
        else:
            profile = self.profileClass()

        self.runner = Runner(binary=self.binary, cmdargs=cmdargs, profile=profile)
        self.runner.start()
        return True

    def stop(self):
        self.runner.stop()

    def wait(self):
        self.runner.wait()

class ThunderbirdNightly(Nightly):
    appName = 'thunderbird'
    name = 'thunderbird'
    profileClass = ThunderbirdProfile

    def getRepoName(self, date):
        # sneaking this in here
        if mozinfo.os == "win" and date < datetime.date(2010, 03, 18):
           # no .zip package for Windows, can't use the installer
           print "Can't run Windows builds before 2010-03-18"
           sys.exit()
           # XXX this should throw an exception vs exiting without the error code

        if date < datetime.date(2008, 7, 26):
            return "trunk"
        elif date < datetime.date(2009, 1, 9):
            return "comm-central"
        elif date < datetime.date(2010, 8, 21):
            return "comm-central-trunk"
        else:
            return "comm-central"


class FirefoxNightly(Nightly):
    appName = 'firefox'
    name = 'firefox'
    profileClass = FirefoxProfile

    def getRepoName(self, date):
        if date < datetime.date(2008, 6, 17):
            return "trunk"
        else:
            return "mozilla-central"

class FennecNightly(Nightly):
    appName = 'fennec'
    name = 'fennec'
    profileClass = FirefoxProfile
    buildRegex = 'fennec-.*\.apk'
    binary = 'org.mozilla.fennec/.App'
    bits = None

    def __init__(self, repo_name=None, bits=mozinfo.bits, persist=None):
        self.repo_name = repo_name
        self.persist = persist
        if "y" != raw_input("WARNING: bisecting nightly fennec builds will clobber your existing nightly profile. Continue? (y or n)"):
            raise Exception("Aborting!")

    def getRepoName(self, date):
        return "mozilla-central-android"

    def install(self):
        subprocess.check_call(["adb", "uninstall", "org.mozilla.fennec"])
        subprocess.check_call(["adb", "install", self.dest])
        return True

    def start(self, profile, addons, cmdargs):
        subprocess.check_call(["adb", "shell", "am start -n %s" % self.binary])
        return True

    def stop(self):
        # TODO: kill fennec (don't really care though since uninstalling it kills it)
        # PID = $(adb shell ps | grep org.mozilla.fennec | awk '{ print $2 }')
        # adb shell run-as org.mozilla.fennec kill $PID
        return True

    def getAppInfo(self):
        archive = zipfile.ZipFile(self.dest, 'r')
        f = archive.open('application.ini')
        parser = ConfigParser()
        parser.readfp(f)
        try:
            changeset = parser.get('App', 'SourceStamp')
            repo = parser.get('App', 'SourceRepository')
            return (repo, changeset)
        except:
            return None

class NightlyRunner(object):

    apps = {'thunderbird': ThunderbirdNightly,
            'fennec': FennecNightly,
            'firefox': FirefoxNightly}

    def __init__(self, addons=None, appname="firefox", repo_name=None,
                 profile=None, cmdargs=(), bits=mozinfo.bits, persist=None):
        self.app = self.apps[appname](repo_name=repo_name, bits=bits, persist=persist)
        self.addons = addons
        self.profile = profile
        self.persist = persist
        self.cmdargs = list(cmdargs)

    def install(self, date=datetime.date.today()):
        if not self.app.download(date=date):
            print "Could not find build from %s" % date
            return False # download failed
        print "Installing nightly"
        return self.app.install()

    def start(self, date=datetime.date.today()):
        if not self.install(date):
            return False
        print "Starting nightly"
        if not self.app.start(self.profile, self.addons, self.cmdargs):
            return False
        return True

    def stop(self):
        self.app.stop()

    def wait(self):
        self.app.wait()

    def cleanup(self):
        self.app.cleanup()

    def getAppInfo(self):
        return self.app.getAppInfo()

def parseBits(optionBits):
    """returns the correctly typed bits"""
    if optionBits == "32":
        return 32
    else:
        # if 64 bits is passed on a 32 bit system, it won't be honored
        return mozinfo.bits

def cli(args=sys.argv[1:]):
    """moznightly command line entry point"""

    # parse command line options
    parser = OptionParser()
    parser.add_option("-d", "--date", dest="date", help="date of the nightly",
                      metavar="YYYY-MM-DD", default=str(datetime.date.today()))
    parser.add_option("-a", "--addons", dest="addons",
                      help="list of addons to install",
                      metavar="PATH1,PATH2")
    parser.add_option("-p", "--profile", dest="profile", help="path to profile to user", metavar="PATH")
    parser.add_option("-n", "--app", dest="app", help="application name",
                      type="choice",
                      metavar="[%s]" % "|".join(NightlyRunner.apps.keys()),
                      choices=NightlyRunner.apps.keys(),
                      default="firefox")
    parser.add_option("-r", "--repo", dest="repo_name", help="repository name on ftp.mozilla.org",
                      metavar="[tracemonkey|mozilla-1.9.2]", default=None)
    parser.add_option("--bits", dest="bits", help="force 32 or 64 bit version (only applies to x86_64 boxes)",
                      choices=("32","64"), default=mozinfo.bits)
    parser.add_option("--persist", dest="persist", help="the directory in which files are to persist ie. /Users/someuser/Documents")
    options, args = parser.parse_args(args)

    options.bits = parseBits(options.bits)

    # XXX https://github.com/mozilla/mozregression/issues/50
    addons = strsplit(options.addons or "", ",")

    # run nightly
    runner = NightlyRunner(appname=options.app, addons=addons,
                           profile=options.profile, repo_name=options.repo_name, bits=options.bits,
                           persist=options.persist)
    runner.start(get_date(options.date))
    try:
        runner.wait()
    except KeyboardInterrupt:
        runner.stop()


if __name__ == "__main__":
    cli()



import re
import urllib2
import json
from utils import urlLinks
import mozinfo
import sys
from optparse import OptionParser

def getBuildBaseURL(appName='firefox', bits=mozinfo.bits):

    if appName == 'fennec':
        return "http://inbound-archive.pub.build.mozilla.org/pub/mozilla.org/mobile/tinderbox-builds/mozilla-inbound-android/"

    baseURL = 'http://inbound-archive.pub.build.mozilla.org/pub/mozilla.org/firefox/tinderbox-builds/'
    if mozinfo.os == "win":
        if bits == 64:
            # XXX this should actually throw an error to be consumed by the caller
            print "No builds available for 64 bit Windows (try specifying --bits=32)"
            sys.exit()
        else:
            return baseURL + 'mozilla-inbound-win32/'
    elif mozinfo.os == "linux":
        if bits == 64:
            return baseURL + 'mozilla-inbound-linux64/'
        else:
            return baseURL + 'mozilla-inbound-linux/'
    elif mozinfo.os == "mac":
        return baseURL + 'mozilla-inbound-macosx64/'

def getInboundRevisions(startRev, endRev, appName='firefox', bits=mozinfo.bits):

    revisions = []
    r = urllib2.urlopen('https://hg.mozilla.org/integration/mozilla-inbound/'
                        'json-pushes?fromchange=%s&tochange=%s'% (startRev,
                                                                  endRev))
    pushlog = json.loads(r.read())
    for pushid in sorted(pushlog.keys()):
        push = pushlog[pushid]
        revisions.append((push['changesets'][-1], push['date']))

    revisions.sort(key=lambda r: r[1])
    if not revisions:
        return []
    starttime = revisions[0][1]
    endtime = revisions[-1][1]
    rawRevisions = map(lambda l: l[0], revisions)

    baseURL = getBuildBaseURL(appName=appName, bits=bits)
    range = 60*60*4 # anything within four hours is potentially within the range
    timestamps = map(lambda l: int(l.get('href').strip('/')),
                     urlLinks(baseURL))
    timestampsInRange = filter(lambda t: t > (starttime - range) and
                               t < (endtime + range), timestamps)
    revisions = [] # timestamp, order pairs
    for timestamp in timestampsInRange:
        for link in urlLinks("%s%s/" % (baseURL, timestamp)):
            href = link.get('href')
            if re.match('.*txt', href):
                url = "%s%s/%s" % (baseURL, timestamp, href)
                contents = urllib2.urlopen(url).read()
                remoteRevision = None
                for line in contents.splitlines():
                    parts = line.split('/rev/')
                    if len(parts) == 2:
                        remoteRevision = parts[1]
                        break
                if remoteRevision:
                    for (i, revision) in enumerate(rawRevisions):
                        if remoteRevision in revision:
                            revisions.append((revision, timestamp, i))

    return sorted(revisions, key=lambda r: r[2])

def cli(args=sys.argv[1:]):

    parser = OptionParser()
    parser.add_option("--start-rev", dest="startRev", help="start revision")
    parser.add_option("--end-rev", dest="endRev", help="end revision")

    options, args = parser.parse_args(args)
    if not options.startRev or not options.endRev:
        print "start revision and end revision must be specified"
        sys.exit(1)

    revisions = getInboundRevisions(options.startRev, options.endRev)
    print "Revision, Timestamp, Order"
    for revision in revisions:
        print ", ".join(map(lambda s: str(s), revision))

if __name__ == "__main__":
    cli()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import mozinfo
import sys
import datetime
from utils import download_url, urlLinks
import re

class NightlyDownloader(object):

    @staticmethod
    def _getBuildRegex(bits):
        if mozinfo.os == "win":
            if bits == 64:
                # XXX this should actually throw an error to be consumed by the caller
                print "No builds available for 64 bit Windows"
                sys.exit()
            return ".*win32.zip"
        elif mozinfo.os == "linux":
            if bits == 64:
                return ".*linux-x86_64.tar.bz2"
            else:
                return ".*linux-i686.tar.bz2"
        elif mozinfo.os == "mac":
            return ".*mac.*\.dmg"

    def __init__(self, repo_name=None, bits=mozinfo.bits, cacheDir=None):
        self.buildRegex = self._getBuildRegex(bits)
        self.cacheDir = cacheDir
        self.repo_name = repo_name

    def getBuildUrl(self, datestamp):
        url = "http://ftp.mozilla.org/pub/mozilla.org/" + self.appName + "/nightly/"
        year = str(datestamp.year)
        month = "%02d" % datestamp.month
        day = "%02d" % datestamp.day
        repo_name = self.repo_name or self.getRepoName(datestamp)
        url += year + "/" + month + "/"

        linkRegex = '^' + year + '-' + month + '-' + day + '-' + '[\d-]+' + repo_name + '/$'
        cachekey = year + '-' + month
        if cachekey in self._monthlinks:
            monthlinks = self._monthlinks[cachekey]
        else:
            monthlinks = urlLinks(url)
            self._monthlinks[cachekey] = monthlinks

        # first parse monthly list to get correct directory
        for dirlink in monthlinks:
            dirhref = dirlink.get("href")
            if re.match(linkRegex, dirhref):
                # now parse the page for the correct build url
                for link in urlLinks(url + dirhref):
                    href = link.get("href")
                    if re.match(self.buildRegex, href):
                        return url + dirhref + href

    def download(self, date=datetime.date.today(), dest=None):
        url = self.getBuildUrl(date)
        if url:
            if not dest:
                dest = self.get_destination(url, date)
            if not self.persist:
                self.remove_lastdest()

            self.dest = self.lastdest = dest
            download_url(url, dest)
            return True
        else:
            return False



from utils import urlLinks, strsplit, get_date
import re
import sys
from runnightly import FennecNightly, FirefoxNightly, NightlyRunner, parseBits
from inboundfinder import getBuildBaseURL
import mozinfo
from optparse import OptionParser

class FirefoxInbound(FirefoxNightly):

    repo_name = None

    def __init__(self, bits=mozinfo.bits, persist=None):
        self.persist = persist
        self.buildRegex = self._getBuildRegex(bits)
        self.bits = bits

    def getBuildUrl(self, timestamp):
        url = "%s%s/" % (getBuildBaseURL(self.bits), timestamp)
        for link in urlLinks(url):
            href = link.get("href")
            if re.match(self.buildRegex, href):
                return url + href

    def getRepoName(self, date):
        return "mozilla-inbound"

class FennecInbound(FennecNightly):

    repo_name = None

    def __init__(self, persist=None):
        self.persist = persist

    def getBuildUrl(self, timestamp):
        url = "%s%s/" % (getBuildBaseURL(appName=self.appName), timestamp)
        for link in urlLinks(url):
            href = link.get("href")
            if re.match(self.buildRegex, href):
                return url + href

    def getRepoName(self, date):
        return "mozilla-inbound"

class InboundRunner(NightlyRunner):

    def __init__(self, addons=None, appname="firefox", repo_name=None,
                 profile=None, cmdargs=(), bits=mozinfo.bits, persist=None):
        if appname == 'firefox':
            self.app = FirefoxInbound(bits=bits, persist=persist)
        else:
            self.app = FennecInbound(persist=persist)
        self.appName = appname
        self.bits = bits
        self.addons = addons
        self.profile = profile
        self.persist = persist
        self.cmdargs = list(cmdargs)

def cli(args=sys.argv[1:]):

    parser = OptionParser()
    parser.add_option("--timestamp", dest="timestamp", help="timestamp of "
                      "inbound build")
    parser.add_option("-a", "--addons", dest="addons",
                      help="list of addons to install",
                      metavar="PATH1,PATH2")
    parser.add_option("-p", "--profile", dest="profile", help="path to profile to user", metavar="PATH")
    parser.add_option("--bits", dest="bits", help="force 32 or 64 bit version (only applies to x86_64 boxes)",
                      choices=("32","64"), default=mozinfo.bits)
    parser.add_option("--persist", dest="persist", help="the directory in which files are to persist ie. /Users/someuser/Documents")

    options, args = parser.parse_args(args)
    if not options.timestamp:
        print "timestamp must be specified"
        sys.exit(1)
    options.bits = parseBits(options.bits)
    # XXX https://github.com/mozilla/mozregression/issues/50
    addons = strsplit(options.addons or "", ",")
    runner = InboundRunner(addons=addons, profile=options.profile,
                           bits=options.bits, persist=options.persist)
    runner.start(get_date(options.date))
    try:
        runner.wait()
    except KeyboardInterrupt:
        runner.stop()

if __name__ == "__main__":
    cli()



#!/usr/bin/env python

import datetime
import time
from utils import urlLinks
import re
import urllib2

baseURL = 'http://inbound-archive.pub.build.mozilla.org/pub/mozilla.org/firefox/tinderbox-builds/mozilla-inbound-linux/'

timestamps = map(lambda l: int(l.get('href').strip('/')), urlLinks(baseURL))
start = (datetime.datetime(2013, 11, 6), '70de5e24d79b')
end = (datetime.datetime(2013, 11, 8), '70f21fad60a4')
range = 60*60*4 # anything within four hours is potentially within the range
valid_timestamps = filter(lambda t: t > (time.mktime(start[0].timetuple()) - range) and t < (time.mktime(end[0].timetuple()) + range), timestamps)
for timestamp in valid_timestamps:
    for link in urlLinks("%s%s/" % (baseURL, timestamp)):
        href = link.get('href')
        if re.match('.*txt', href):
            url = "%s%s/%s" % (baseURL, timestamp, href)
            contents = urllib2.urlopen(url).read()
            rev = None
            for line in contents.splitlines():
                parts = line.split('/rev/')
                if len(parts) == 2:
                    rev = parts[1]
                    break
            if rev:
                print (datetime.datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d %H:%M:%S'), rev)




import os
from setuptools import setup

desc = """Regression range finder for Mozilla nightly builds"""

# take description from README
here = os.path.dirname(os.path.abspath(__file__))
try:
    summ = file(os.path.join(here, 'README.md')).read()
except:
    summ = """Interactive regression range finder for Mozilla nightly builds"""

setup(name="mozregression",
      version="0.11",
      description=desc,
      long_description=summ,
      author='Mozilla Automation and Tools Team',
      author_email='tools@lists.mozilla.org',
      url='http://github.com/mozilla/mozregression',
      license='MPL 1.1/GPL 2.0/LGPL 2.1',
      packages=['mozregression'],
      entry_points="""
          [console_scripts]
          mozregression = mozregression:regressioncli
          moznightly = mozregression:nightlycli
        """,
      platforms =['Any'],
      install_requires = ['httplib2 >= 0.6.0',
                          'mozcommitbuilder >= 0.4.10',
                          'mozfile >= 0.1',
                          'mozprofile >= 0.4',
                          'mozrunner >= 5.14',
                          'BeautifulSoup >= 3.0.4',
                          'mozinstall >= 1.4',
                          'mozinfo >= 0.4'
                          ],
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Console',
                   'Intended Audience :: Developers',
                   'Operating System :: OS Independent'
                  ]
     )
